package GetStatusListing;

###############################################################################
# LSF
###############################################################################

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(get_status_listing);

use Reporting
  qw(debug_report failed_report ok_report start_report command_report report_and_die);

use strict;

# Queries the BSS for the state of (at least) all UNICORE jobs
# and returns a list of their identifier together with the
# status (and optionally the queue name)
#
# Called in response to a #TSI_JOBQUERY
#
# No args
#
# Returns void to TSI
#         fail/success to NJS and a list of jobs
#
#  First line is "QSTAT",
#  followed by a line per found job, first word is BSS job identifier
#  and the second word is one of RUNNING, QUEUED, SUSPENDED or COMPLETED
#  Optionally, a third word can be added which is the queue name
#
#  The list must include all UNICORE jobs (but can contain  extra jobs)
#
sub get_status_listing {

    my ( $command, $output, $result, @lines, $bssid, $state, $ustate, $queue );

    debug_report("Finding all jobs on BSS");

    # Form the request command.
    $command = "$main::qstat_cmd";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if ( $? != 0 ) {
        failed_report($output);
    }
    else {

# The output format of the interesting lines is:
# xxxxxx user Status Queue ....
#
# Where xxxxx is the Job id to return, the number of
# junk fields is variable 
# Example:
# JOBID USER STAT QUEUE FROM_HOST EXEC_HOST JOB_NAME SUBMIT_TIME
# 1652  bob  DONE normal gridnode1  gridnode3   date Jun 19 12:45
#
# There may be also some uninteresting lines (not starting with a numerical
# batch system id)

        $result = "QSTAT\n";
        $_      = $output;

        # we want to extract the numeric job ID, the status and the queue name
        while ( /^\s*(\d+)\s+\S+\s+(\w+)\s+(\w+)\.*/gm )
        {

            $bssid = $1; 
	    $state = $2; 
	    $queue = $3;

	    $ustate = "UNKNOWN";

	    $ustate = "QUEUED" if $state =~ /PEND/; 
	    $ustate = "QUEUED" if $state =~ /WAIT/; 
	    $ustate = "QUEUED" if $state =~ /ZOMBI/;

	    $ustate = "RUNNING" if $state =~ /EXIT/; 
	    $ustate = "RUNNING" if $state =~ /RUN/;

	    $ustate = "SUSPENDED" if $state =~ /PSUSP/;
	    $ustate = "SUSPENDED" if $state =~ /USUSP/;
	    $ustate = "SUSPENDED" if $state =~ /SSUSP/;

	    $ustate = "COMPLETED" if $state =~ /EXIT/; 
            $ustate = "COMPLETED" if $state =~ /DONE/;

	    $result = "$result $bssid $ustate $queue\n";

	}

        print main::CMD_SOCK $result;
        debug_report("qstat executed OK");
    }
}

#
#                   Copyright (c) Fujitsu Ltd 2000 - 2004
#
#                Use and distribution is subject a License.
# A copy was supplied with the distribution (see documentation or the jar file).
#
# This product includes software developed by Fujitsu Limited (http://www.fujitsu.com).
